#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""

    Software name: BayesPol
    Version: v.0.1
        

    Created on Mon Jun 20 17:44:23 2016


    @author:    Diego Herranz
                herranz@ifca.unican.es
                

"""

import numpy as np
import matplotlib.pyplot as plt
import logpriors_pol as lp
import emcee
import corner
from scipy.stats import mstats


"""
     Observation parameters:
         
         For this test version of the code, we are not using QUIJOTE-specific
         instrumental characteristics. Pixel size, image size, observation 
         frequency, FWHMs and noise levels should be modified accordingly
         to the instrumental specifications of the experiment under study.
         
"""

pix_size     = 3.5355             # pixel size, in arcmin
t_obs        = 100                # bumber of samples (observations) per pixel
size         = 32                 # image size, in pixels
freqs        = [20.0,8.6,4.8]     # observing frequencies
fwhms        = np.array([36.0,55.2,55.2])/pix_size      # in pixel units
nlevels      = np.array([0.3,0.2,0.18])/np.sqrt(t_obs)  # in Jy


"""
     MCMC parameters:
"""

ndim     = 5      # number of parameters in the model
nwalkers = 50     # number of MCMC walkers
nburn    = 5000   # "burn-in" period to let chains stabilize
nsteps   = 10000  # number of MCMC steps to take

"""
     Environment parameters:  
"""                     
root_dir     = '/Users/herranz/Trabajo/Bayes/Polarization/MCMC/Results_Python/'  # working directory



"""
    --------------------------------------------------------------------------
    
              Multi - Frequency Bayesian Detection/Estimation
              of polarized point sources
              
    --------------------------------------------------------------------------

"""



def run_sampler(output_index,prob_type,intensidades,parametros,data):
    
"""
    Runs a MCMC sampler on a data cube DATA (dimension = SIZE*SIZE*(2*FREQS.size))
    and writes to a file the MonteCarlo traces, estimated polarization parameters
    and their confidence limits 
    
    The input DATA is a numpy array arranged in the following manner:
        [Xpixel,Ypixel,Polfreq], where the Polfreq dimension is ordered 
        as [Q[0],Q[0],Q[1],U[1],...,Q[freq-1],U[nfreq-1]], and where
        nfreq = freqs.size
    

"""
   

    starting_guesses = np.random.random((nwalkers, ndim))
    starting_guesses[:,0] = 5*starting_guesses[:,0]
    starting_guesses[:,1] = 5*starting_guesses[:,1]
    starting_guesses[:,2] = 5*starting_guesses[:,2]

    if prob_type=='posterior':
        sampler = emcee.EnsembleSampler(nwalkers, ndim, lp.log_posterior, args=[data,size,intensidades,freqs,fwhms,nlevels])
    else:
        sampler = emcee.EnsembleSampler(nwalkers, ndim, lp.log_posterior_fake, args=[data,size,intensidades,freqs,fwhms,nlevels])

    smp = sampler.run_mcmc(starting_guesses, nsteps)

    emcee_trace = sampler.chain[:, nburn:, :].reshape(-1, ndim).T
    emcee_trace = np.transpose(emcee_trace)

    params_cls = np.zeros((5,3))

    for i in list(range(5)):
        L = emcee_trace[:,i]
        A = mstats.mquantiles(L, prob=[0.16, 0.5, 0.84])
        params_cls[i,:] = A

    Cmat = np.cov(emcee_trace,rowvar=False)

    nombre = root_dir+'output_'+prob_type+'_{0}.npz'.format(output_index)
    np.savez_compressed(nombre,data=data,emcee_trace=emcee_trace,Cmat=Cmat,
                        intensidades=intensidades,parametros=parametros,
                        params_cls=params_cls)

    fig = corner.corner(emcee_trace,
                           labels=[r"$\Pi_1$ (%)", r"$\Pi_2$ (%)",
                                   r"$\Pi_3$ (%)", r"$\rho$ (rad)",
                                   r"$\psi_0$ (rad)"],
                           show_titles=True,
                           quantiles=[0.16, 0.5, 0.84])
    nombre_figura = root_dir+'figura_'+prob_type+'_{0}.pdf'.format(output_index)
    fig.savefig(nombre_figura)
    nombre_figura = root_dir+'figura_'+prob_type+'_{0}.jpg'.format(output_index)
    fig.savefig(nombre_figura)
    plt.close(fig)
    sampler.reset()

    return

